@@ -46,6 +46,18 @@ module Network.TLS.State
     , getSession
     , isSessionResuming
     , isClientContext
+    , setExporterMasterSecret
+    , getExporterMasterSecret
+    , setTLS13KeyShare
+    , getTLS13KeyShare
+    , setTLS13PreSharedKey
+    , getTLS13PreSharedKey
+    , setTLS13HRR
+    , getTLS13HRR
+    , setTLS13Cookie
+    , getTLS13Cookie
+    , setClientSupportsPHA
+    , getClientSupportsPHA
     -- * random
     , genRandom
     , withRNG
@@ -53,8 +65,9 @@ module Network.TLS.State
 
 import Network.TLS.Imports
 import Network.TLS.Struct
+import Network.TLS.Struct13
 import Network.TLS.RNG
-import Network.TLS.Types (Role(..))
+import Network.TLS.Types (Role(..), HostName)
 import Network.TLS.Wire (GetContinuation)
 import Network.TLS.Extension
 import qualified Data.ByteString as B
@@ -63,8 +76,6 @@ import Network.TLS.ErrT
 import Crypto.Random
 import Data.X509 (CertificateChain)
 
-type HostName = String
-
 data TLSState = TLSState
     { stSession             :: Session
     , stSessionResuming     :: Bool
@@ -74,6 +85,7 @@ data TLSState = TLSState
     , stExtensionALPN       :: Bool  -- RFC 7301
     , stHandshakeRecordCont :: Maybe (GetContinuation (HandshakeType, ByteString))
     , stNegotiatedProtocol  :: Maybe B.ByteString -- ALPN protocol
+    , stHandshakeRecordCont13 :: Maybe (GetContinuation (HandshakeType13, ByteString))
     , stClientALPNSuggest   :: Maybe [B.ByteString]
     , stClientGroupSuggest  :: Maybe [Group]
     , stClientEcPointFormatSuggest :: Maybe [EcPointFormat]
@@ -82,6 +94,12 @@ data TLSState = TLSState
     , stRandomGen           :: StateRNG
     , stVersion             :: Maybe Version
     , stClientContext       :: Role
+    , stTLS13KeyShare       :: Maybe KeyShare
+    , stTLS13PreSharedKey   :: Maybe PreSharedKey
+    , stTLS13HRR            :: !Bool
+    , stTLS13Cookie         :: Maybe Cookie
+    , stExporterMasterSecret :: Maybe ByteString -- TLS 1.3
+    , stClientSupportsPHA   :: !Bool -- Post-Handshake Authentication (TLS 1.3)
     }
 
 newtype TLSSt a = TLSSt { runTLSSt :: ErrT TLSError (State TLSState) a }
@@ -106,15 +124,22 @@ newTLSState rng clientContext = TLSState
     , stServerVerifiedData  = B.empty
     , stExtensionALPN       = False
     , stHandshakeRecordCont = Nothing
+    , stHandshakeRecordCont13 = Nothing
     , stNegotiatedProtocol  = Nothing
     , stClientALPNSuggest   = Nothing
-    , stClientGroupSuggest = Nothing
+    , stClientGroupSuggest  = Nothing
     , stClientEcPointFormatSuggest = Nothing
     , stClientCertificateChain = Nothing
     , stClientSNI           = Nothing
     , stRandomGen           = rng
     , stVersion             = Nothing
     , stClientContext       = clientContext
+    , stTLS13KeyShare       = Nothing
+    , stTLS13PreSharedKey   = Nothing
+    , stTLS13HRR            = False
+    , stTLS13Cookie         = Nothing
+    , stExporterMasterSecret = Nothing
+    , stClientSupportsPHA   = False
     }
 
 updateVerifiedData :: Role -> ByteString -> TLSSt ()
@@ -236,3 +261,39 @@ withRNG f = do
     let (a,rng') = withTLSRNG (stRandomGen st) f
     put (st { stRandomGen = rng' })
     return a
+
+setExporterMasterSecret :: ByteString -> TLSSt ()
+setExporterMasterSecret key = modify (\st -> st { stExporterMasterSecret = Just key })
+
+getExporterMasterSecret :: TLSSt (Maybe ByteString)
+getExporterMasterSecret = gets stExporterMasterSecret
+
+setTLS13KeyShare :: Maybe KeyShare -> TLSSt ()
+setTLS13KeyShare mks = modify (\st -> st { stTLS13KeyShare = mks })
+
+getTLS13KeyShare :: TLSSt (Maybe KeyShare)
+getTLS13KeyShare = gets stTLS13KeyShare
+
+setTLS13PreSharedKey :: Maybe PreSharedKey -> TLSSt ()
+setTLS13PreSharedKey mpsk = modify (\st -> st { stTLS13PreSharedKey = mpsk })
+
+getTLS13PreSharedKey :: TLSSt (Maybe PreSharedKey)
+getTLS13PreSharedKey = gets stTLS13PreSharedKey
+
+setTLS13HRR :: Bool -> TLSSt ()
+setTLS13HRR b = modify (\st -> st { stTLS13HRR = b })
+
+getTLS13HRR :: TLSSt Bool
+getTLS13HRR = gets stTLS13HRR
+
+setTLS13Cookie :: Maybe Cookie -> TLSSt ()
+setTLS13Cookie mcookie = modify (\st -> st { stTLS13Cookie = mcookie })
+
+getTLS13Cookie :: TLSSt (Maybe Cookie)
+getTLS13Cookie = gets stTLS13Cookie
+
+setClientSupportsPHA :: Bool -> TLSSt ()
+setClientSupportsPHA b = modify (\st -> st { stClientSupportsPHA = b })
+
+getClientSupportsPHA :: TLSSt Bool
+getClientSupportsPHA = gets stClientSupportsPHA
