@@ -5,18 +5,25 @@ module Network.TLS.Util
         , partition3
         , partition6
         , fromJust
-        , and'
         , (&&!)
         , bytesEq
         , fmapEither
         , catchException
+        , forEitherM
+        , mapChunks_
+        , getChunks
+        , Saved
+        , saveMVar
+        , restoreMVar
         ) where
 
+import qualified Data.ByteArray as BA
 import qualified Data.ByteString as B
 import Network.TLS.Imports
 
 import Control.Exception (SomeException)
 import Control.Concurrent.Async
+import Control.Concurrent.MVar
 
 sub :: ByteString -> Int -> Int -> Maybe ByteString
 sub b offset len
@@ -52,10 +59,6 @@ fromJust :: String -> Maybe a -> a
 fromJust what Nothing  = error ("fromJust " ++ what ++ ": Nothing") -- yuck
 fromJust _    (Just x) = x
 
--- | This is a strict version of and
-and' :: [Bool] -> Bool
-and' l = foldl' (&&!) True l
-
 -- | This is a strict version of &&.
 (&&!) :: Bool -> Bool -> Bool
 True  &&! True  = True
@@ -67,12 +70,41 @@ False &&! False = False
 -- it's a non lazy version, that will compare every bytes.
 -- arguments with different length will bail out early
 bytesEq :: ByteString -> ByteString -> Bool
-bytesEq b1 b2
-    | B.length b1 /= B.length b2 = False
-    | otherwise                  = and' $ B.zipWith (==) b1 b2
+bytesEq = BA.constEq
 
 fmapEither :: (a -> b) -> Either l a -> Either l b
 fmapEither f = fmap f
 
 catchException :: IO a -> (SomeException -> IO a) -> IO a
 catchException action handler = withAsync action waitCatch >>= either handler return
+
+forEitherM :: Monad m => [a] -> (a -> m (Either l b)) -> m (Either l [b])
+forEitherM []     _ = return (pure [])
+forEitherM (x:xs) f = f x >>= doTail
+  where
+    doTail (Right b) = fmap (b :) <$> forEitherM xs f
+    doTail (Left e)  = return (Left e)
+
+mapChunks_ :: Monad m
+           => Int -> (B.ByteString -> m a) -> B.ByteString -> m ()
+mapChunks_ len f = mapM_ f . getChunks len
+
+getChunks :: Int -> B.ByteString -> [B.ByteString]
+getChunks len bs
+    | B.length bs > len =
+        let (chunk, remain) = B.splitAt len bs
+         in chunk : getChunks len remain
+    | otherwise = [bs]
+
+-- | An opaque newtype wrapper to prevent from poking inside content that has
+-- been saved.
+newtype Saved a = Saved a
+
+-- | Save the content of an 'MVar' to restore it later.
+saveMVar :: MVar a -> IO (Saved a)
+saveMVar ref = Saved <$> readMVar ref
+
+-- | Restore the content of an 'MVar' to a previous saved value and return the
+-- content that has just been replaced.
+restoreMVar :: MVar a -> Saved a -> IO (Saved a)
+restoreMVar ref (Saved val) = Saved <$> swapMVar ref val
