@@ -23,6 +23,7 @@ module Network.TLS.X509
     , FailedReason
     , ServiceID
     , wrapCertificateChecks
+    , pubkeyType
     ) where
 
 import Data.X509
@@ -41,6 +42,7 @@ data CertificateRejectReason =
           CertificateRejectExpired
         | CertificateRejectRevoked
         | CertificateRejectUnknownCA
+        | CertificateRejectAbsent
         | CertificateRejectOther String
         deriving (Show,Eq)
 
@@ -56,4 +58,9 @@ wrapCertificateChecks l
     | Expired `elem` l   = CertificateUsageReject   CertificateRejectExpired
     | InFuture `elem` l  = CertificateUsageReject   CertificateRejectExpired
     | UnknownCA `elem` l = CertificateUsageReject   CertificateRejectUnknownCA
+    | SelfSigned `elem` l = CertificateUsageReject  CertificateRejectUnknownCA
+    | EmptyChain `elem` l = CertificateUsageReject  CertificateRejectAbsent
     | otherwise          = CertificateUsageReject $ CertificateRejectOther (show l)
+
+pubkeyType :: PubKey -> String
+pubkeyType = show . pubkeyToAlg
